function [dists,times] = SpectroMeterDists(Sinput)

%% prepare shape
S.VERT = Sinput.X; S.TRIV = Sinput.T;
S.M = sparse(Sinput.Ae); S.S = Sinput.cotLaplacian;
S.phi = Sinput.laplaceBasis; S.lambda = Sinput.eigenvalues;
S.Y = S.VERT(:,2); S.Z = S.VERT(:,3); S.X = S.VERT(:,1);
S.nv = Sinput.nv; S.nf = Sinput.nf;

tOps = tic;
[S.GradOp,S.DivOp,S.grad_lbo,S.grad_lbo_RQt] = GetGradDivOp(S);
tOps = toc(tOps);

u0 = sparse(1:S.nv,1:S.nv,ones(S.nv,1),S.nv,S.nv);

tEDG = tic;
S.edge_len = EdgeLengthStats(S);
tEDG = toc(tEDG);

tArea = tic;
S.tri_area = calc_tri_areas(S);
tArea = toc(tArea);

%% run SpectroMeter (note I use non-deault time value)
tSpectral = tic;
dists = SpectroMeter(S,u0,20.3e-3);
dists = 0.5*(dists+dists');
tSpectral = toc(tSpectral);
times = tOps + tEDG + tArea + tSpectral;

end

function el = EdgeLengthStats(shape)
%%
edges = [
    shape.TRIV(:,[1 2])
    shape.TRIV(:,[1 3])
    shape.TRIV(:,[2 3])    ];
edges = unique([edges ; fliplr(edges)],'rows');

vert = [shape.X shape.Y shape.Z];
edge_len = sqrt(sum((...
    vert(edges(:,1),:) - ...
    vert(edges(:,2),:)).^2,2));

el.avg = mean(  edge_len);
el.min = min(   edge_len);
el.max = max(   edge_len);
el.med = median(edge_len);
end


function S_tri = calc_tri_areas(M)
%%
getDiff  = @(a,b)M.VERT(M.TRIV(:,a),:) - M.VERT(M.TRIV(:,b),:);
getTriArea  = @(X,Y).5*sqrt(sum(cross(X,Y).^2,2));
S_tri = getTriArea(getDiff(1,2),getDiff(1,3));


end
